{
  PATCH5ED.EXE by Frank Fischer 7/11/91

  Pascal source and Exe files given to the public domain.

  Fixes DOS 5.0 QBASIC.EXE so that EDIT.COM will automatically
  show all files in the directory in the "open file" window.

  Works better than another patcher, DOS5ED.EXE, which makes the
  mistake of leaving a blank after the "*.*" filespec.

}

{$A+,B-,D+,E+,F-,I+,L+,N-,O-,R-,S+}
{$M $4000,0,0}
{$V-}

PROGRAM patch5ed;

  uses
    crt, dos;

  VAR
    qbasfile  : file;
    qbasnew   : file;
    readbuf   : array[1..60000] of char;
    numread, numwritten : word;
    i         : integer;
    tempfile  : file;
    dirinfo   : searchrec;
    ch: char;
    del_qbasbac : boolean;

FUNCTION fileexists(FileName: string) : Boolean;
  { Returns True if file exists; otherwise,
    it returns False. Closes the file if
    it exists. }
  var
    f: file;
  BEGIN
    {$I-}
    Assign(f, FileName);
    Reset(f);
    Close(f);
    {$I+}
    FileExists := (IOResult = 0) and
      (FileName <> '');
  END;  { fileexists }

PROCEDURE write_intro;
  BEGIN
    clrscr;
    writeln;
    writeln('             * DOS 5.0 EDIT Patch by Frank Fischer *');
    writeln('Turbo Pascal 6.0 source and EXE file given to the public domain.');
    writeln;
    writeln('This patch utility will change the DOS 5.0 EDIT default filespec');
    writeln('from "*.TXT" to "*.*".  This utility will make a new copy of QBASIC.EXE');
    writeln('(which DOS 5.0 uses as an editor), and rename the original QBASIC.BAC.');
    writeln('The new copy of QBASIC.EXE created will not require you to backspace');
    writeln('over "TXT" to get a file listing of all files in the current directory.');
    writeln('This program must be run in the same directory that QBASIC.EXE resides in.');
    writeln;
    writeln('There exists a program in the public domain which incorrectly changes');
    writeln('the "*.TXT" filespec to "*.* " instead of "*.*".  If you have already');
    writeln('used the other patcher, this program will patch the already patched');
    writeln('copy of QBASIC.EXE.  The problem with the other patcher is that it');
    writeln('will not allow the DOS 5.0 editor to change directories unless you');
    writeln('first backspace over the blank.');
    writeln;
  END; {write_intro}

PROCEDURE look_for_qbasic;
  BEGIN
    findfirst('QBASIC.EXE',archive,dirinfo);
    IF doserror <> 0 THEN
      BEGIN
        writeln('QBASIC.EXE not found in current directory.');
        write  ('Patch ABORTED');
        halt;
      END;
  END; {look_for_qbasic}

PROCEDURE check_for_orig_size;
  BEGIN
    IF dirinfo.size <> 254799 THEN
      BEGIN
        writeln('QBASIC.EXE should be 254799 bytes in size.');
        writeln('The file in your directory named QBASIC.EXE is ',
          dirinfo.size,' bytes long.');
        write  ('Patch ABORTED');
        halt;
      END;
  END; {check_for_orig_size}

PROCEDURE abort_patch;
  BEGIN
    write  ('Patch ABORTED.');
    halt;
  END;

PROCEDURE ask_for_permission;
  BEGIN
    write  ('Hit the ENTER key to patch QBASIC.EXE, or any other key to abort...');
    ch := readkey;
    writeln;
    IF ch = #0 THEN
      BEGIN
        ch := readkey;
        abort_patch;
      END
    ELSE
      IF ch <> #13 THEN abort_patch;
  END; {ask_for_permission}

PROCEDURE patch_error;
  BEGIN
    writeln('Unexpected disk error.');
    writeln('Your copy of QBASIC.EXE has been corrupted by this program.');
    writeln('You will have to copy QBASIC.BAC to QBASIC.EXE.');
    writeln('Program halted.');
    halt;
  END; {patch_error}

PROCEDURE do_the_patch;
  BEGIN
    assign(qbasfile,'QBASIC.EXE');
    IF fileexists('QBASIC.BAC') THEN
      BEGIN
        writeln('QBASIC.BAC already exists in your directory.');
        write  ('Overwrite the file (Y/N)?');
        ch := readkey;
        writeln;
        IF ch = #0 THEN
          ch := readkey;
        IF upcase(ch) = 'Y' THEN
          BEGIN
            assign(tempfile,'QBASIC.BAC');
            erase(tempfile);
            rename(qbasfile,'QBASIC.BAC');
            writeln('QBASIC.EXE renamed to QBASIC.BAC.');
            del_qbasbac := false;
          END
        ELSE
          BEGIN
            rename(qbasfile,'$t$e$m$p.$pp');
            writeln('QBASIC.BAC backup of QBASIC.EXE not created.');
            del_qbasbac := true;
          END;
      END
    ELSE
      BEGIN
        rename(qbasfile,'QBASIC.BAC');
        writeln('QBASIC.EXE renamed to QBASIC.BAC.');
        del_qbasbac := false;
      END;
    assign(qbasnew,'QBASIC.EXE');
    rewrite(qbasnew,1);
    reset(qbasfile,1);
    FOR i := 1 to 4 DO
      BEGIN
        blockread(qbasfile, readbuf, sizeof(readbuf), numread);
        IF numread <> sizeof(readbuf) THEN patch_error;
        blockwrite(qbasnew, readbuf, numread, numwritten);
        IF numread <> numwritten THEN patch_error;
      END;
    blockread(qbasfile, readbuf, sizeof(readbuf), numread);
    IF (readbuf[7408] <> '*') or (readbuf[7409] <> '.') THEN 
      BEGIN
        writeln('Your copy of QBASIC.EXE was corrupt to start with.');
        writeln('It is likely that the patched copy will not work.');
        writeln('Patch continuing anyway.');
      END;
    readbuf[7408] := '*';
    readbuf[7409] := '.';
    IF readbuf[7410] <> 'T' THEN
      BEGIN
        writeln('Your copy of QBASIC.EXE has already been patched.');
        writeln('Continuing with new patch anyway.');
      END;
    readbuf[7410] := '*';
    readbuf[7411] := chr(0);
    readbuf[7412] := chr(0);
    blockwrite(qbasnew, readbuf, numread, numwritten);
    close(qbasfile);
    IF del_qbasbac THEN erase(qbasfile);
    close(qbasnew);
    write  ('Patched copy of QBASIC.EXE successfully created.');
  END; {do_the_patch}


BEGIN {main block}
  assign(input,'');  reset(input);
  assign(output,''); rewrite(output);
  write_intro;
  look_for_qbasic;
  check_for_orig_size;
  ask_for_permission;
  do_the_patch;
END. {main block}
